
/**
 * Title:        ASCII File Adapter<p>
 * Description:  This package contains all non JCA classes of the
 * ASCII file adapter.<p>
 * Copyright:    Copyright (c) Atul Apte<p>
 * Company:      iConexio Technologies Inc.<p>
 * @author Atul Apte
 * @version 1.0
 */
package com.iconexio.asciiFileAdapter;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.Vector;

public class ASCIIDataFileParser extends Object implements FileParser
{

  public ASCIIDataFileParser()
  {
    this.pEngine = new ASCIIDataFileParserEngine();
  }

  public ASCIIDataFileParser(ASCIIDataFileParserEngine theEngine)
  {
    this.pEngine = theEngine;
  }

  public void setFieldDelimiter(byte fieldDelim)
  {
    this.pEngine.setFieldDelimiter(fieldDelim);
  }

  public void processFile(String fileName) throws IOException, ParserException,
                                                  Exception
  {
    FileInputStream fis;
    byte[] rec;
    int n, r;

    fis = new FileInputStream(fileName);
    try
    {
      while (true)
      {
        n = fis.available();
        if (n <= 0)
          break;
        if (n > 10241)
        {
          Exception e = new Exception("File too big.. > 10K");
          throw e;
        }
        if (n > 0)
        {
          // There is data to be read, create a byte array to hold the data
          rec = new byte[n];
          r = fis.read(rec);
          if (r == -1)
            break; // break out of the while loop
          try
          {
            pEngine.parseBuffer(rec,n);
          }
          catch (ParserException e)
          {
             throw e;
          }
        } // End of if n > 0
      } // End of while loop
    } catch (IOException e) { throw e; }
  }

  public Vector getRecordContainer()
  {
    // Call the ASCIIDataFileParserEngine.getRecordContainer
    return(pEngine.getRecordContainer());
  }
  // Member variables
  private ASCIIDataFileParserEngine pEngine;
}
