
/**
 * Title:        ASCII File Adapter<p>
 * Description:  This package contains all non JCA classes of the
 * ASCII file adapter.<p>
 * Copyright:    Copyright (c) Atul Apte<p>
 * Company:      iConexio Technologies Inc.<p>
 * @author Atul Apte
 * @version 1.0
 */
package com.iconexio.asciiFileAdapter;

import java.util.Vector;

/*
 * This is the API for all client services offered by the ASCII File
 * resource adapter. Since the contents of the ASCII file can change
 * their structure the adapter services accept meta-data
 * information in a different file.
 * All customer objects created as a result of the
 * process are stored in a Vector instance.
*/

public interface ClientAPI
{

  // This API uses the default meta data file name set by the configuration
  // interface. If there is not default metaData file name then this API
  // returns an exception. The meta data file contains the file name of the
  // customer data ASCII file
  // Pre-condition: The calling method must check for the files existence
  // The implementation of this method assumes the file is accessible and
  // readable
  public void extractRecords() throws Exception;
  // This API requires the invoking method do define a specific meta-data
  // and customer data file name
  // Pre-condition: The calling method must check for the files existence and
  //             and if its readable
  public void extractRecords(String dataFileName, String metadataFileName)
                                          throws Exception;
  // This API returns the Vector holding the parsed customer record objects.
  // objects.
  // Pre-condition: Either one of the processASCIIFile methods has been
  //                successfully executed.
  public Vector getCustomerRecordList();
  // This API checks the validity of the customer record
  public boolean isCustomerRecordValid(CustomerRec aRec);
}