/* Generated by Together */
/* The LogFileManager provides a class for managing file based logs
 * This class must be extended to manage specific types of log files
 *
 * @author Atul Apte
 * @version 1.0
 */

package com.iconexio.asciiFileAdapter;

import java.io.*;
import java.util.*;
import java.lang.*;

abstract public class LogFileManager extends Object implements LogManager {
    public LogFileManager() {
        // The default constructor allows for only one log file
        // to be open at any given time
        _noOfLogFilesOpen = 0;
        _maxLogFilesOpen = 2;
        logFileTable = new Hashtable();
    }
    public LogFileManager(int maxNoOfLogFiles) {
        // If you need more then one log files open at the same
        // time use this constructor. The actual maximum files open
        // in any environment is defined by the system kernel.
        _maxLogFilesOpen = maxNoOfLogFiles;
        _noOfLogFilesOpen = 0;
        logFileTable = new Hashtable();
    }

    // mode can be a string with the value WRITE or APPEND
    public void openLog(String theLogFileName, String mode) throws
        TooManyLogFilesOpenException, Exception {
        // Check to see if there is any room to open more files
	    if (getNoOfLogFilesOpen() >=  getMaxNoOfLogFiles()) {
		TooManyLogFilesOpenException e = new
                    TooManyLogFilesOpenException("Cannot open " +
                      theLogFileName + "Too many log files open");
            throw e;
          }
       	// Open a log file and store the handle in a hash table

       	File logFile = new File(theLogFileName);
       	FileOutputStream logFileStream;
       	if (mode.equalsIgnoreCase("WRITE") == true) {
               try {
                  // Open file in write mode
                  logFileStream = new
                          FileOutputStream(theLogFileName);
               } catch (Exception e) {
                  throw e;
                 }
               logFileTable.put(theLogFileName, logFileStream);
           	   incrementNoOfFilesOpen();
        	}
	      else if (mode.equalsIgnoreCase("APPEND") == true) {
               try {
                  // Open file in append mode
                  logFileStream = new
                       FileOutputStream(theLogFileName, true);
		   } catch (Exception e) {
                    throw e;
                 }
            logFileTable.put(theLogFileName, logFileStream);
        	incrementNoOfFilesOpen();
        	}
            else {
                // Throw an exception
                TooManyLogFilesOpenException e = new
                     TooManyLogFilesOpenException("File " +
                         theLogFileName +
                         " exists. Cannot open in WRITE mode");
                throw e;
            }
    }

    public void openLog(String theLogFileName, String mode,
            String userID, String password)
            throws TooManyLogFilesOpenException, Exception {
        openLog(theLogFileName,mode);
    }

    public void closeLog(String theFileName) throws IOException {
        Object logFileStream = (Object)logFileTable.remove(theFileName);
        if (logFileStream != null) {
           Class logClass = logFileStream.getClass();
    	   String className = logClass.getName();
               if (className.equalsIgnoreCase("FileOutputStream") == true) {
                  FileOutputStream theStream = (FileOutputStream)logFileStream;
                  try {
		    theStream.close();
                  } catch (IOException e) {
                    throw e;
                  }
	       }
        }
    }

    public void logMessage(String logName, String logMsg) throws Exception {
        // Retrieve the correct file stream object matching the log name
        FileOutputStream theFileStream;
        theFileStream = (FileOutputStream)logFileTable.get(logName);
        if (theFileStream == null) {
            NullPointerException e = new
                NullPointerException("Cannot find matching file stream for "
                    + logName);
            throw e;
        }
        byte[] msgInBytes = logMsg.getBytes();
        theFileStream.write(msgInBytes);
        theFileStream.write((int)10);
        theFileStream.write((int)13);
    }

    public int getNoOfLogFilesOpen() {
        return(_noOfLogFilesOpen);
    }

    public int getMaxNoOfLogFiles() {
        return(_maxLogFilesOpen);
    }

    public void incrementNoOfFilesOpen() {
        _noOfLogFilesOpen++;
    }

    public void decrementNoOfFilesOpen() {
        _noOfLogFilesOpen--;
    }

    private int _noOfLogFilesOpen;
    private int _maxLogFilesOpen;
    private Hashtable logFileTable;
    private final char nl = '\n';
}
