
/**
 * Title:        ASCII File Adapter<p>
 * Description:  This package contains all non JCA classes of the
 * ASCII file adapter.<p>
 * Copyright:    Copyright (c) Atul Apte<p>
 * Company:      iConexio Technologies Inc.<p>
 * @author Atul Apte
 * @version 1.0
 */

package com.iconexio.asciiFileAdapter;

public class TagValueParserStateMachine
{
  // The default constructor of this class resets the parser state
  public TagValueParserStateMachine()
  {
        resetParser();
  }

  // This method returns true if the tag represents start of an element
  public boolean startTagDetected()
  {
    if (currentState == endTag && tagType == prefixTag)
      return(true);
    return(false);
  }
  // This method returns true if the tag represents end of an element
  public boolean endTagDetected()
  {
    if (currentState == endTag && tagType == suffixTag)
      return(true);
    return(false);
  }
  // This method returns true if the parsing of element value is complete
  public boolean tagValueDetected()
  {
    if (currentState == processingDataComplete)
      return(true);
    return(false);
  }
  // This method just resets the data buffer
  public void resetDataBuffer()
  {
    dataIterator = 0;
  }
  // This method returns the string representing the parsed data
  public String getData()
  {
    String s;
    if (dataIterator == 0)
      return((String)null); // No data has been parsed or the statemachine
                            // has been reset.
    s = new String(parsedData,0,dataIterator);
    return(s);
  }

  public void processByte(byte a) throws ParserException {
    switch (currentState)
    {
      case idleState:
        if (a == tagSDelim) // Start of a element tag is detected
          currentState = startTag; // Change state
        // Else ignore all bytes until start of tag is detected
      break;

      case startTag:
        if (a == tagEDelim) // Tag end delimiter detected
        {
          currentState = endTag; // End of tag is detected
          break;
        }
        else
        {
          // Store the bytes as tag name
          if (dataIterator >= 32)
          {
            // Tag cannot be bigger then 32 bytes long
            ParserException e = new
                            ParserException("Tag name cannot exceed 32 bytes");
            throw(e);
          }
          if (a == '/' && dataIterator == 0)
            tagType = suffixTag; // Do not store the byte
          else
          {
            // Store the byte as data
            parsedData[dataIterator] = a;
            dataIterator++;
          }
        }
      break;

      case endTag:
          if (a == tagSDelim) {
             currentState = startTag;
             break;
          }
          if (a == EOR) // This indicates end of a record in the buffer
          {
              currentState = idleState;
              tagType = prefixTag;
              break;
          }
          parsedData[dataIterator] = a;
          dataIterator++;
          currentState = processingData;
       break;

       case processingData:

         if (a == tagSDelim)
         {
            // Start of end tag detected
            currentState = processingDataComplete;
         }
         else
         {
            parsedData[dataIterator] = a;
            dataIterator++;
         }

       break;

       case processingDataComplete:

         if (a == tagEDelim) {
            // Null tag name detected throw exception
            ParserException e = new
                 ParserException("End Tag naming error expecting /");
            throw e;
         }
         else {
            if (a == '/')
               currentState = startTag;
               tagType = suffixTag;
         }
       break;
    }
    return;
  }

  // This method resets the entire parser state including allocating a new
  // buffer. The maximum data size is set to 1K
  public void resetParser()
  {
       currentState = idleState;
       parsedData = new byte[1025];
       for (int i = 0; i <= 1024; i++)
           parsedData[i] = 0;
       tagType = prefixTag;
       dataIterator = 0;
       setTagSDelim((byte)'<');
       setTagEDelim((byte)'>');
  }

  public void setTagSDelim(byte a) {
        tagSDelim = a;
  }
  public void setTagEDelim(byte a) {
        tagEDelim = a;
  }
  public byte getTagEDelim() {
        return(tagEDelim);
  }
  public byte getTagSDelim() {
        return(tagSDelim);
  }

    // Parser state engine member variables

    private byte[] parsedData;
    private int dataIterator;

    private byte tagSDelim;
    private byte tagEDelim;

    private int currentState;
    private int tagType; // prefixTag, suffixTag

    // Parser states
    private final static int idleState = 0;
    private final static int startTag = 1;
    private final static int endTag = 2;
    private final static int processingData = 3;
    private final static int processingDataComplete = 4;
    private final static byte EOR = 13;

    // Tag types
    private final static int prefixTag = 1;
    private final static int suffixTag = 2;
}
