
/**
 * Title:        ASCII File Resource Adapter<p>
 * Description:  This project is about building a JCA 1.0 compatible
 * resource adapter as part of the book on developing
 * JCA adapters.<p>
 * Copyright:    Copyright (c) Atul Apte<p>
 * Company:      iConexio Technologies Inc.<p>
 * @author Atul Apte
 * @version 1.0
 */
package com.iConexio.jca.asciiFileRA;

import java.io.*;
import java.io.Serializable;
import javax.resource.Referenceable;
import javax.resource.*;
import javax.resource.spi.*;
import javax.naming.Reference;
import javax.resource.cci.*;

public class AFConnectionFactory implements
                   javax.resource.cci.ConnectionFactory,
                   java.io.Serializable, javax.resource.Referenceable
{

    public AFConnectionFactory(ManagedConnectionFactory mcf,
                          ConnectionManager cm)
    {
        this.mcf = mcf;
        if (cm == null)
        {
            this.cm = new AFConnectionManager(); // This is the default
                                                 // Connection Manager
        }
        else
        {
            this.cm = cm;
        }
    }

    public AFConnectionFactory(ManagedConnectionFactory mcf)
    {
        this.mcf = mcf;
    }

    public javax.resource.cci.Connection getConnection()
      throws ResourceException
    {
      javax.resource.cci.Connection con = null;
      // Delegate the getConnection function to connection manager
      // and its allocate connection method
      con =
        (javax.resource.cci.Connection) cm.allocateConnection(mcf, null);
	return con;
    }

    public javax.resource.cci.Connection
            getConnection(ConnectionSpec properties) throws ResourceException
    {
      javax.resource.cci.Connection con = null;
      // Create a connection request info object from connection properties
      // This connection request info object is sent to the managed connection
      AFConnectionRequestInfo info =
            new AFConnectionRequestInfo(
		    ((AFConnectionSpec)properties).getUser(),
		    ((AFConnectionSpec)properties).getPassword(),
                ((AFConnectionSpec)properties).getASCIIFileName(),
                ((AFConnectionSpec)properties).getMetadataFileName());
        con = (javax.resource.cci.Connection)
            cm.allocateConnection(mcf,info);
	return con;
    }

    public ASCIIFileMetaData getMetaData() throws ResourceException
    {
        return new ASCIIFileAdapterMetaData();
    }

    public RecordFactory getRecordFactory() throws ResourceException
    {
        throw new ResourceException("Record Factory not supported");
    }

    public void setReference(Reference reference)
    {
        this.reference = reference;
    }

    public Reference getReference()
    {
        return reference;
    }

    // Private variables
    private ManagedConnectionFactory mcf;
    private ConnectionManager cm;
    private Reference reference;
}
