
/**
 * Title:        ASCII File Resource Adapter<p>
 * Description:  This project is about using existing Java
 * application level classes and encapsulating them
 * in a JCA 1.0 compliant resource adapter<p>
 * Copyright:    Copyright (c) Atul Apte<p>
 * Company:      iConexio Technologies Inc.<p>
 * @author Atul Apte
 * @version 1.0
 */
package com.iConexio.jca.asciiFileRA;

import javax.resource.ResourceException;
import javax.resource.spi.*;
import javax.resource.*;
import javax.resource.spi.security.PasswordCredential;
import java.io.Serializable;
import javax.resource.spi.security.PasswordCredential;
import javax.resource.spi.SecurityException;
import java.io.*;
import javax.security.auth.Subject;
import java.util.*;
import javax.naming.Context;
import javax.naming.InitialContext;

/** The AFManagedConnectionFactory instance create instances of
 * AFConnectionFactory as well as ASCIIFileManagedConnection
 */
public class AFManagedConnectionFactory implements ManagedConnectionFactory,
                                                   Serializable
{

  public AFManagedConnectionFactory()
  {
    factoryID = new String("ASCII File");
  }

  /** This method creates a connection factory instance. The parameter
   * connection manager is passed by the application server. The constructor
   * for the connection factory also accepts a managed connection factory
   * instance for future reference. The connection manager instance is there to
   * support connection pooling by the application server.
   */
   public Object createConnectionFactory(ConnectionManager cm) throws
                    ResourceException
   {
      AFConnectionFactory afcf;
      afcf = new AFConnectionFactory(this, cm);
      return(afcf);
   }

   /** This method creates a connection factory instance without any
    * associated connectio manager. The application server does not perform
    * any connection pooling when connections are retrieved from this type
    * of connection factory later.
    */
    public Object createConnectionFactory() throws ResourceException
    {
       AFConnectionFactory afcf;
       afcf = new AFConnectionFactory(this, null);
       return(afcf);
    }

    public ManagedConnection
        createManagedConnection(Subject subject,
                                ConnectionRequestInfo info)
        throws ResourceException
    {

        String userName = null;
        PasswordCredential pc =
            Util.getPasswordCredential(this, subject, info);
        if (pc != null) {
            userName = pc.getUserName();
        }
        return new ASCIIFileManagedConnection(this,pc);
    }

    public ManagedConnection
            matchManagedConnections(Set connectionSet,
                                    Subject subject,
                                    ConnectionRequestInfo info)
        throws ResourceException
    {

        PasswordCredential pc =
            Util.getPasswordCredential(this, subject, info);
        Iterator it = connectionSet.iterator();
        while (it.hasNext())
        {
            Object obj = it.next();
            if (obj instanceof ASCIIFileManagedConnection)
            {
                ASCIIFileManagedConnection mc =
                                     (ASCIIFileManagedConnection) obj;
                ManagedConnectionFactory mcf =
                    mc.getManagedConnectionFactory();
                if (Util.isPasswordCredentialEqual
                    (mc.getPasswordCredential(), pc) &&
                    mcf.equals(this))
                {
                    AFConnectionRequestInfo afcri =
                               (AFConnectionRequestInfo)info;
                    String s = afcri.getASCIIFileName();
                    if (mc.getASCIIFileName().equalsIgnoreCase(s) == true)
                       return mc;
                }
            }
        }
        return null;
    }

    public boolean equals(Object obj) {
        if (obj == null) return false;
        if (obj instanceof AFManagedConnectionFactory) {
            String v1 = ((AFManagedConnectionFactory) obj).
                getFactoryID();
            String v2 = this.getFactoryID();
            return (v1 == null) ? (v2 == null) : (v1.equals(v2));
        } else {
            return false;
        }
    }

    public int hashCode() {
        if (getFactoryID() == null) {
            return (new String("")).hashCode();
        } else {
            return getFactoryID().hashCode();
        }
    }

    private transient Context ctx;
    private String factoryID;
}
