
/**
 * Title:        ASCII File Adapter<p>
 * Description:  This package contains all non JCA classes of the
 * ASCII file adapter.<p>
 * Copyright:    Copyright (c) Atul Apte<p>
 * Company:      iConexio Technologies Inc.<p>
 * @author Atul Apte
 * @version 1.0
 */
package com.iconexio.asciiFileAdapter;

/**
 * The test harness is a standalong Java program that uses the ASCII file
 * parser classes to parse the customer data file and display the contents
 * to the standard output.
 */
import java.util.Vector;
import java.lang.Integer;
import java.io.File;

public class testHarness
{
  public testHarness()
  {
    LogManager lm;
    LogManagerFactory lmf;
    int badRec;
    badRec = 0;

    // Create a log manager for exception log files
    lmf = new LogManagerFactory();
    lm = lmf.createLogManager("EXCEPTION");

    try
    {
      lm.openLog("BADREC.DAT","APPEND");
    } catch(Exception e)
    {
      System.err.println(e.getMessage());
    }
    try
    {
      lm.setTimestamp("BADREC.DAT");
    } catch(Exception e)
    {
      System.err.println(e.getMessage());
    }

    ClientAPIImpl cAPI = new ClientAPIImpl();
    try
    {
      cAPI.extractRecords();
    } catch(Exception e)
    {
      // Log the exception in a log file
      try
      {
        lm.logMessage("BADREC.DAT",e.getMessage());
      } catch(Exception e1)
      {
        System.err.println(e1.getMessage());
      }
    }

    // Open the statistics log file
    try
    {
      lm.openLog("PSTATS.DAT","WRITE");
    } catch(Exception e)
    {
      System.err.println(e.getMessage());
    }

    Vector custRecs;
    custRecs = cAPI.getCustomerRecordList();
    CustomerRec cRec;
    int z;
    z = custRecs.size();
    for (int i = 0; i < z; i++)
    {
      cRec = (CustomerRec)custRecs.get(i);
      if (cAPI.isCustomerRecordValid(cRec) == true)
      {
        System.out.println(cRec.getID());
        System.out.println(cRec.getName());
        System.out.println(cRec.getRecAction());
        System.out.println(cRec.getAddress());
        System.out.println(cRec.getStatus());
        System.out.println(cRec.getOutstandingBal());
      }
      else
        badRec++; // Keep a counter for recording number of invalid records
    }

    // Log statistics before closing the log files
    Integer zz = new Integer(z);
    Integer yy = new Integer(badRec);
    // Write this information to the PSTATS.DAT log file
    try
    {
      lm.logMessage("PSTATS.DAT",zz.toString());
      lm.logMessage("PSTATS.DAT",yy.toString());
    } catch(Exception e1)
    {
      System.err.println(e1.getMessage());
    }

    try
    {
      lm.closeLog("BADREC.DAT");
      lm.closeLog("PSTATS.DAT");
    } catch(Exception e)
    {
      System.err.println(e.getMessage());
    }
  }


  /** The parameter metaDataFileName references the meta data definitions
   *  and the parameter customerDataFileName references the actual customer
   *  data file name
   */
  public testHarness(String metaDataFileName, String customerDataFileName)
  {
    // Since we are testing the same data files the pstats.log file is not
    // present in this method
    LogManager lm;
    LogManagerFactory lmf = new LogManagerFactory();
    lm = lmf.createLogManager("EXCEPTION");
    try
    {
      lm.openLog("BADREC.DAT","APPEND");
    } catch(Exception e)
    {
      System.err.println(e.getMessage());
    }
    try
    {
      lm.setTimestamp("BADREC.DAT");
    }
    catch(Exception e)
    {
      System.err.println(e.getMessage());
    }

    ClientAPIImpl cAPI = new ClientAPIImpl();

    try
    {
      cAPI.extractRecords(customerDataFileName,metaDataFileName);
    }
    catch(Exception e)
    {
      // Log the error in a log file
      try
      {
        lm.logMessage("BADREC.DAT",e.getMessage());
      }
      catch(Exception e1)
      {
        System.err.println(e1.getMessage());
      }
    }

    Vector custRecs;
    custRecs = cAPI.getCustomerRecordList();

    CustomerRec cRec;
    int z;
    z = custRecs.size();
    for (int i = 0; i < z; i++)
    {
      cRec = (CustomerRec)custRecs.get(i);
      if (cAPI.isCustomerRecordValid(cRec) == true)
      {
        System.out.println(cRec.getID());
        System.out.println(cRec.getName());
        System.out.println(cRec.getRecAction());
        System.out.println(cRec.getAddress());
        System.out.println(cRec.getStatus());
        System.out.println(cRec.getOutstandingBal());
      }
    }
    try
    {
      lm.closeLog("BADREC.DAT");
    }
    catch(Exception e)
    {
      System.err.println(e.getMessage());
    }
  }

  public static void main(String[] args)
  {
    if (args.length == 0)
    {
      // First parse the default customer data file as defined in the
      // default meta data definition file METADATA.DAT
      testHarness testHarness1 = new testHarness();
      // Print the total number of records processed
      AdminAPIImpl a = new AdminAPIImpl();
      try
      {
        System.out.println("No of record processed = " +
                   a.getNoOfRecordsProcessed());
      } catch(Exception e)
      {
        System.err.println(e.getMessage());
      }
      return;
    }
    if (args.length == 2)
    {
      // Next parse the customer data file and meta data file passed to this
      // program as arguments
      File testArg;
      testArg = new File(args[0]);
      if (testArg.exists() == false)
      {
        System.out.println("File name specified in args[0] " + args[0] +
              " Does not exist");
        return;
      }
      testArg = new File(args[1]);
      if (testArg.exists() == false)
      {
        System.out.println("File name specified in args[0] " + args[1] +
              " Does not exist");
        return;
      }

      testHarness testHarness2 = new testHarness(args[0], args[1]);
      return;
    }
    if (args.length > 2 || args.length == 1)
    {
      System.out.println(
               "Usage 1: java com.iconexio.asciiFileAdapter.testHarness");
      System.out.println(
               "Usage 2: java com.iconexio.asciiFileAdapter.testHarness " +
               "<Meta data file name> <Cutomer data file name>");
      return;
    }
  }
}